//------------------------------------------------------------------------------
// ピンボールコントローラー　切替機能付き
//                                                          したびらめ(Solefish)
//                                                    http://shitabirame.sub.jp/
//------------------------------------------------------------------------------
// 履歴
// ver 1.0  2013/07/06  初出
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// 機能のON/OFF
//------------------------------------------------------------------------------
// ONにしたい機能の#define行頭の//を外してください。

// デバッグモード
//#define DEBUG

// アナログ入力によるナッジ操作を行う
#define USE_ANALOG_NUDGE

// デジタル入力によるナッジ操作を行う
//#define USE_DIGITAL_NUDGE

// ディップスイッチによる機能切り替えを行う
#define USE_DIPSWITCH


//------------------------------------------------------------------------------
// キー割り当て
//------------------------------------------------------------------------------
// ゲームのキー設定をここに記述してください。
//   割り振りたいキーをシングルクォーテーションで囲って記述します。（例：'1'）
//   英字キーを割り当てるときは、小文字で指定します。（例：'f'）
//   特殊キーを割り当てるときは、Arduinoスケッチ用定数を記述してください。
//     http://arduino.cc/en/Reference/KeyboardModifiers
// 注意：右SHIFTキーなど一部のキーはゲームが認識できない事があります。
//       その場合はあらかじめゲームのキー設定で認識できるキーに設定を変更してください。
//
// 定義名    対応する入力
// KB_ST_n   前面左ボタン（実機のスタートボタンの位置）
// KB_PL_n   前面右ボタン（実機のプランジャーの位置）
// KB_FL_n   左側面ボタン（実機の左フリッパーの位置）
// KB_FR_n   右側面ボタン（実機の右フリッパーの位置）
// KB_NL_n   左ナッジ（左から力を加えて台を揺らす）
// KB_NR_n   右ナッジ（右から力を加えて台を揺らす）
// KB_NU_n   上ナッジ（下から力を加えて台を揺らす）
//
// ゲーム側の設定を変更している箇所には設定名の欄に*を付けておくと分かりやすくなります。
//
// Windowsのショートカットキーと重ならないような設定をおすすめします。
// 例：Ctrl+Escを押すと、デスクトップに戻ってスタートメニューが開いてしまいます

// ディップスイッチが 0(0000) の時の設定
// または、ディップスイッチを実装しない時の設定
#define SET_NAME_0          "Worms Pinball"     // 設定名
//      定義名              送信するキー           機能
#define KB_ST_0             KEY_ESC             // ポーズ
#define KB_PL_0             KEY_RETURN          // スタート/プランジャー
#define KB_FL_0             'y'                 // 左フリッパー*
#define KB_FR_0             'n'                 // 右フリッパー*
#define KB_NL_0             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_0             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_0             ' '                 // 上ナッジ

// ディップスイッチが 1(0001) の時の設定
#define SET_NAME_1          "Pinball FX2"       // 設定名
//      定義名              送信するキー           機能
#define KB_ST_1             KEY_ESC             // ポーズ
#define KB_PL_1             KEY_RETURN          // プランジャー
#define KB_FL_1             KEY_LEFT_ARROW      // 左フリッパー*
#define KB_FR_1             KEY_RIGHT_ARROW     // 右フリッパー*
#define KB_NL_1             KEY_RIGHT_CTRL      // 左ナッジ（左右逆）
#define KB_NR_1             KEY_LEFT_CTRL       // 右ナッジ（左右逆）
#define KB_NU_1             ' '                 // 上ナッジ

// ディップスイッチが 2(0010) の時の設定
#define SET_NAME_2          "Pure Pinball 2.0"  // 設定名
//      定義名              送信するキー           機能
#define KB_ST_2             's'                 // スタート
#define KB_PL_2             KEY_RETURN          // プランジャー
#define KB_FL_2             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_2             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_2             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_2             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_2             ' '                 // 上ナッジ

// ディップスイッチが 3(0011) の時の設定
#define SET_NAME_3          "Dream Pinball 3D"  // 設定名
//      定義名              送信するキー           機能
#define KB_ST_3             KEY_ESC             // ポーズ
#define KB_PL_3             KEY_RETURN          // スタート/プランジャー
#define KB_FL_3             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_3             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_3             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_3             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_3             ' '                 // 上ナッジ

// ディップスイッチが 4(0100) の時の設定
#define SET_NAME_4          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_4             KEY_ESC             // ポーズ
#define KB_PL_4             KEY_RETURN          // プランジャー
#define KB_FL_4             KEY_LEFT_ARROW      // 左フリッパー
#define KB_FR_4             KEY_RIGHT_ARROW     // 右フリッパー
#define KB_NL_4             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_4             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_4             ' '                 // 上ナッジ

// ディップスイッチが 5(0101) の時の設定
#define SET_NAME_5          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_5             KEY_ESC             // ポーズ
#define KB_PL_5             KEY_RETURN          // プランジャー
#define KB_FL_5             KEY_LEFT_ARROW      // 左フリッパー
#define KB_FR_5             KEY_RIGHT_ARROW     // 右フリッパー
#define KB_NL_5             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_5             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_5             ' '                 // 上ナッジ

// ディップスイッチが 6(0110) の時の設定
#define SET_NAME_6          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_6             KEY_ESC             // ポーズ
#define KB_PL_6             KEY_RETURN          // プランジャー
#define KB_FL_6             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_6             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_6             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_6             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_6             ' '                 // 上ナッジ

// ディップスイッチが 7(0111) の時の設定
#define SET_NAME_7          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_7             KEY_ESC             // ポーズ
#define KB_PL_7             KEY_RETURN          // プランジャー
#define KB_FL_7             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_7             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_7             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_7             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_7             ' '                 // 上ナッジ

// ディップスイッチが 8(1000) の時の設定
#define SET_NAME_8          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_8             KEY_ESC             // ポーズ
#define KB_PL_8             KEY_RETURN          // プランジャー
#define KB_FL_8             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_8             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_8             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_8             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_8             ' '                 // 上ナッジ

// ディップスイッチが 9(1001) の時の設定
#define SET_NAME_9          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_9             KEY_ESC             // ポーズ
#define KB_PL_9             KEY_RETURN          // プランジャー
#define KB_FL_9             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_9             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_9             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_9             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_9             ' '                 // 上ナッジ

// ディップスイッチが A(1010) の時の設定
#define SET_NAME_A          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_A             KEY_ESC             // ポーズ
#define KB_PL_A             KEY_RETURN          // プランジャー
#define KB_FL_A             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_A             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_A             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_A             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_A             ' '                 // 上ナッジ

// ディップスイッチが B(1011) の時の設定
#define SET_NAME_B          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_B             KEY_ESC             // ポーズ
#define KB_PL_B             KEY_RETURN          // プランジャー
#define KB_FL_B             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_B             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_B             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_B             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_B             ' '                 // 上ナッジ

// ディップスイッチが C(1100) の時の設定
#define SET_NAME_C          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_C             KEY_ESC             // ポーズ
#define KB_PL_C             KEY_RETURN          // プランジャー
#define KB_FL_C             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_C             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_C             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_C             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_C             ' '                 // 上ナッジ

// ディップスイッチが D(1101) の時の設定
#define SET_NAME_D          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_D             KEY_ESC             // ポーズ
#define KB_PL_D             KEY_RETURN          // プランジャー
#define KB_FL_D             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_D             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_D             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_D             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_D             ' '                 // 上ナッジ

// ディップスイッチが E(1110) の時の設定
#define SET_NAME_E          "undefined"         // 設定名
//      定義名              送信するキー           機能
#define KB_ST_E             KEY_ESC             // ポーズ
#define KB_PL_E             KEY_RETURN          // プランジャー
#define KB_FL_E             KEY_LEFT_CTRL       // 左フリッパー
#define KB_FR_E             KEY_RIGHT_CTRL      // 右フリッパー
#define KB_NL_E             KEY_LEFT_ALT        // 左ナッジ
#define KB_NR_E             KEY_RIGHT_ALT       // 右ナッジ
#define KB_NU_E             ' '                 // 上ナッジ

// ディップスイッチが F(1111) の時の設定
#define SET_NAME_F          "TEST"              // 設定名
//      定義名              送信するキー           機能
#define KB_ST_F             '1'                 // テスト 1
#define KB_PL_F             '2'                 // テスト 2
#define KB_FL_F             '3'                 // テスト 3
#define KB_FR_F             '4'                 // テスト 4
#define KB_NL_F             '5'                 // テスト 5
#define KB_NR_F             '6'                 // テスト 6
#define KB_NU_F             '7'                 // テスト 7

//------------------------------------------------------------------------------
// ナッジ（アナログ）の設定
//------------------------------------------------------------------------------

#ifdef USE_ANALOG_NUDGE

// 加速度センサーの初期値
// 加速度センサーが水平で停止している時のAD値を設定します。変更しないでください。
#define SENSE_CENTER_X      511                 // 加速度センサーの水平時X値
#define SENSE_CENTER_Y      511                 // 加速度センサーの水平時Y値

// ナッジ検出しきい値
// 加速度センサーがどれだけ振れたときにナッジが起きるか設定します。
// 0～511の整数を設定してください。値が大きいほどナッジを検出しにくくなります。
#define SENSE_NUDGE_X       191                 // 横ナッジ検出値
#define SENSE_NUDGE_Y       191                 // 縦ナッジ検出値

// ナッジキーホールド時間
// ナッジを検出した時、ナッジキーを送信する時間(ミリ秒)を設定します。
// 1以上の整数を設定してください。
// ゲームによっては時間が短かすぎると反応しない場合があります。
#define NUDGE_TIMESPAN       500                 // ナッジキーを押し続けるフレーム数

#endif

//------------------------------------------------------------------------------
// Arduino入力端子の定義
//------------------------------------------------------------------------------
// 入力端子を変えたときは、ここの設定を変更してください。

//      スケッチ内の名前    端子番号
#define PI_START            0                   // スタートボタン入力端子
#define PI_PLUNGER          1                   // プランジャー入力端子
#define PI_FLIPPER_LEFT     2                   // 左フリッパー入力端子
#define PI_FLIPPER_RIGHT    3                   // 右フリッパー入力端子

#ifdef USE_DIGITAL_NUDGE                        // デジタル入力ナッジを使用する場合
#define PI_NUDGE_LEFT       4                   // 左ナッジ入力端子（デジタル）
#define PI_NUDGE_RIGHT      5                   // 右ナッジ入力端子（デジタル）
#define PI_NUDGE_UP         6                   // 上ナッジ入力端子（デジタル）
#endif

#ifdef USE_ANALOG_NUDGE                         // アナログ入力ナッジを使用する場合
#define PI_NUDGE_X          A0                  // 横ナッジ入力端子（アナログ）
#define PI_NUDGE_Y          A1                  // 縦ナッジ入力端子（アナログ）
#endif

#ifdef USE_DIPSWITCH                            // ディップスイッチを使用する場合
#define PI_DIP_1            13                  // ディップスイッチ1の桁
#define PI_DIP_2            12                  // ディップスイッチ2の桁
#define PI_DIP_4            11                  // ディップスイッチ4の桁
#define PI_DIP_8            10                  // ディップスイッチ8の桁
#endif

//------------------------------------------------------------------------------
// グローバル変数
//------------------------------------------------------------------------------

// キーバッファ
unsigned char KEY_BUFFER[256];                    // キーバッファ
unsigned char KEY_BUFFER_OLD[256];                // 前フレームのキーバッファ

// キー設定配列
// スタート
const int KbArray_Start[16] = 
  { KB_ST_0, KB_ST_1, KB_ST_2, KB_ST_3, KB_ST_4, KB_ST_5, KB_ST_6, KB_ST_7,
    KB_ST_8, KB_ST_9, KB_ST_A, KB_ST_B, KB_ST_C, KB_ST_D, KB_ST_E, KB_ST_F };

// プランジャー
const int KbArray_Plunger[16] = 
  { KB_PL_0, KB_PL_1, KB_PL_2, KB_PL_3, KB_PL_4, KB_PL_5, KB_PL_6, KB_PL_7,
    KB_PL_8, KB_PL_9, KB_PL_A, KB_PL_B, KB_PL_C, KB_PL_D, KB_PL_E, KB_PL_F };

// 左フリッパー
const int KbArray_Flipper_Left[16] = 
  { KB_FL_0, KB_FL_1, KB_FL_2, KB_FL_3, KB_FL_4, KB_FL_5, KB_FL_6, KB_FL_7,
    KB_FL_8, KB_FL_9, KB_FL_A, KB_FL_B, KB_FL_C, KB_FL_D, KB_FL_E, KB_FL_F };

// 右フリッパー
const int KbArray_Flipper_Right[16] = 
  { KB_FR_0, KB_FR_1, KB_FR_2, KB_FR_3, KB_FR_4, KB_FR_5, KB_FR_6, KB_FR_7,
    KB_FR_8, KB_FR_9, KB_FR_A, KB_FR_B, KB_FR_C, KB_FR_D, KB_FR_E, KB_FR_F };

// 左ナッジ
const int KbArray_Nudge_Left[16] = 
  { KB_NL_0, KB_NL_1, KB_NL_2, KB_NL_3, KB_NL_4, KB_NL_5, KB_NL_6, KB_NL_7,
    KB_NL_8, KB_NL_9, KB_NL_A, KB_NL_B, KB_NL_C, KB_NL_D, KB_NL_E, KB_NL_F };

// 右ナッジ
const int KbArray_Nudge_Right[16] = 
  { KB_NR_0, KB_NR_1, KB_NR_2, KB_NR_3, KB_NR_4, KB_NR_5, KB_NR_6, KB_NR_7,
    KB_NR_8, KB_NR_9, KB_NR_A, KB_NR_B, KB_NR_C, KB_NR_D, KB_NR_E, KB_NR_F };

// 上ナッジ
const int KbArray_Nudge_Up[16] = 
  { KB_NU_0, KB_NU_1, KB_NU_2, KB_NU_3, KB_NU_4, KB_NU_5, KB_NU_6, KB_NU_7,
    KB_NU_8, KB_NU_9, KB_NU_A, KB_NU_B, KB_NU_C, KB_NU_D, KB_NU_E, KB_NU_F };


//------------------------------------------------------------------------------
// 初期設定
//------------------------------------------------------------------------------
void setup() {
  
  // デジタル入力ピン設定
  // 使用するデジタルピンを入力に設定し、全てプルアップします。
  pinMode( PI_START        , INPUT_PULLUP );    // スタートボタン入力端子
  pinMode( PI_PLUNGER      , INPUT_PULLUP );    // プランジャー入力端子
  pinMode( PI_FLIPPER_LEFT , INPUT_PULLUP );    // 左フリッパー入力端子
  pinMode( PI_FLIPPER_RIGHT, INPUT_PULLUP );    // 右フリッパー入力端子
  
  #ifdef USE_DIGITAL_NUDGE                      // デジタル入力ナッジを使用する場合
  pinMode( PI_NUDGE_LEFT   , INPUT_PULLUP );    // 左ナッジ入力端子（デジタル）
  pinMode( PI_NUDGE_RIGHT  , INPUT_PULLUP );    // 右ナッジ入力端子（デジタル）
  pinMode( PI_NUDGE_UP     , INPUT_PULLUP );    // 上ナッジ入力端子（デジタル）
  #endif
  
  #ifdef USE_DIPSWITCH                          // ディップスイッチを使用する場合
  pinMode( PI_DIP_1        , INPUT_PULLUP );    // ディップスイッチ1の桁
  pinMode( PI_DIP_2        , INPUT_PULLUP );    // ディップスイッチ2の桁
  pinMode( PI_DIP_4        , INPUT_PULLUP );    // ディップスイッチ4の桁
  pinMode( PI_DIP_8        , INPUT_PULLUP );    // ディップスイッチ8の桁
  #endif
  
  // キーボード機能初期化
  Keyboard.begin();
  
  // 前フレームのキーバッファクリア
  for(int i=0; i<256; i++ ){
  	KEY_BUFFER_OLD[i] = 0;
  }
  
}

//------------------------------------------------------------------------------
// メインループ
//------------------------------------------------------------------------------
void loop() {
  
  // 変数宣言
  #ifdef USE_ANALOG_NUDGE
  static int NudgeBuff_Left  = 0;                        // 左ナッジ残りフレーム数
  static int NudgeBuff_Right = 0;                        // 右ナッジ残りフレーム数
  static int NudgeBuff_Up    = 0;                        // 上ナッジ残りフレーム数
  #endif
  
  // キーバッファクリア
  for(int i=0; i<256; i++ ){
    KEY_BUFFER[i] = 0;
  }
  
  //----------------------------------------------------------------------------
  // ディップスイッチの状態を取得
  //----------------------------------------------------------------------------
  
  int dip = 0;                                  // ディップスイッチの状態
  
  // ディップスイッチが実装されている時は状態を読み取る
  #ifdef USE_DIPSWITCH
  if( digitalRead( PI_DIP_1 ) == LOW ) dip += 1;
  if( digitalRead( PI_DIP_2 ) == LOW ) dip += 2;
  if( digitalRead( PI_DIP_4 ) == LOW ) dip += 4;
  if( digitalRead( PI_DIP_8 ) == LOW ) dip += 8;
  #endif
  
  //----------------------------------------------------------------------------
  // アナログ入力の処理
  //----------------------------------------------------------------------------
  
  #ifdef USE_ANALOG_NUDGE
  
  // アナログ入力読み取り
  int iAccX = analogRead( PI_NUDGE_X );                 // X(左右)加速度値を取得
  int iAccY = analogRead( PI_NUDGE_Y );                 // Y(前後)加速度値を取得
  
  // 左ナッジ（アナログ）
  // ナッジ検出
  if( iAccX > SENSE_CENTER_X + SENSE_NUDGE_X )
    NudgeBuff_Left = NUDGE_TIMESPAN;                    // 左ナッジ残り時間セット
  // ナッジ持続判定
  if( NudgeBuff_Left > 0 ){
    KEY_BUFFER[ KbArray_Nudge_Left[dip] ] = 1;          // キーバッファをON
    NudgeBuff_Left--;                                   // 左ナッジ残り時間を減らす
  }
  
  // 右ナッジ（アナログ）
  // ナッジ検出
  if( iAccX < SENSE_CENTER_X - SENSE_NUDGE_X )
    NudgeBuff_Right = NUDGE_TIMESPAN;                   // 右ナッジ残り時間セット
  // ナッジ持続判定
  if( NudgeBuff_Right > 0 ){
    KEY_BUFFER[ KbArray_Nudge_Right[dip] ] = 1;         // キーバッファをON
    NudgeBuff_Right--;                                  // 右ナッジ残り時間を減らす
  }
  
  // 上ナッジ（アナログ）
  // ナッジ検出
  if( iAccY > SENSE_CENTER_Y + SENSE_NUDGE_Y )
    NudgeBuff_Up = NUDGE_TIMESPAN;                      // 上ナッジ残り時間セット
  // ナッジ持続判定
  if( NudgeBuff_Up > 0 ){
    KEY_BUFFER[ KbArray_Nudge_Up[dip] ] = 1;            // キーバッファをON
    NudgeBuff_Up--;                                     // 上ナッジ残り時間を減らす
  }
  
  #endif
  
  //----------------------------------------------------------------------------
  // デジタル入力の処理
  //----------------------------------------------------------------------------
  
  // スタートボタン
  if( digitalRead( PI_START ) == LOW )
    KEY_BUFFER[ KbArray_Start[dip] ] = 1;               // キーバッファをON
  
  // プランジャーボタン
  if( digitalRead( PI_PLUNGER ) == LOW )
    KEY_BUFFER[ KbArray_Plunger[dip] ] = 1;             // キーバッファをON
  
  // 左フリッパーボタン
  if( digitalRead( PI_FLIPPER_LEFT ) == LOW )
    KEY_BUFFER[ KbArray_Flipper_Left[dip] ] = 1;        // キーバッファをON
  
  // 右フリッパーボタン
  if( digitalRead( PI_FLIPPER_RIGHT ) == LOW )
    KEY_BUFFER[ KbArray_Flipper_Right[dip] ] = 1;       // キーバッファをON
  
  #ifdef USE_DIGITAL_NUDGE
  
  // 左ナッジボタン（デジタル）
  if( digitalRead( PI_NUDGE_LEFT ) == LOW )
    KEY_BUFFER[ KbArray_Nudge_Left[dip] ] = 1;          // キーバッファをON
  
  // 右ナッジボタン（デジタル）
  if( digitalRead( PI_NUDGE_RIGHT ) == LOW )
    KEY_BUFFER[ KbArray_Nudge_Right[dip] ] = 1;         // キーバッファをON
  
  // 上ナッジボタン（デジタル）
  if( digitalRead( PI_NUDGE_UP ) == LOW )
    KEY_BUFFER[ KbArray_Nudge_Up[dip] ] = 1;            // キーバッファをON
  
  #endif
  
  //----------------------------------------------------------------------------
  // キーバッファ送出
  //----------------------------------------------------------------------------
  for( int i=0; i<256; i++ ){
    // キーが新たに押された
    if( KEY_BUFFER[i]==1 && KEY_BUFFER_OLD[i]==0 ) Keyboard.press( i );
    // 押されているキーが離された
    if( KEY_BUFFER[i]==0 && KEY_BUFFER_OLD[i]==1 ) Keyboard.release( i );
    
    // 前フレームのキーバッファの更新
    KEY_BUFFER_OLD[i] = KEY_BUFFER[i];
  }
  
  //----------------------------------------------------------------------------
  // 1ミリ秒待つ
  //----------------------------------------------------------------------------
  delay(1);
  
}
